###Sketch Config
#type = PluginCompound
#class_name = 'Cube'
#menu_text = 'Cube'
###End

import Sketch
from Sketch import Point, SingularMatrix
from Sketch.Graphics import handle
from lib.autoshapelib import AutoShape

width = 20
class Cube(AutoShape):
    class_name = 'Cube'
    p = Point(10,10)

    def GetSnapPoints(self):
        return Point(0,0), Point(width,0), Point(0,width), Point(width,width)
    
    def GetHandles(self):
        p = self.p + Point(width, width)
        return [handle.MakeNodeHandle(self.trafo(p))]
                
    def recompute(self):
        w, h = self.p

        path = Sketch.CreatePath()
        newpaths = [path]
        path.AppendLine((width, 0))
        path.AppendLine((width+w, h))
        path.AppendLine((width+w, h+width))
        path.AppendLine((width, width))
        
        path = Sketch.CreatePath()
        newpaths.append(path)
        path.AppendLine((width, width))
        path.AppendLine((width+w, h+width))
        path.AppendLine((w, h+width))
        path.AppendLine((0, width))

        path = Sketch.CreatePath()
        newpaths.append(path)
        path.AppendLine((0,0))
        path.AppendLine((0, width))
        path.AppendLine((width, width))
        path.AppendLine((width, 0))
        path.AppendLine((0,0))

        for path in newpaths:
            path.Transform(self.trafo)
        if self.objects:
            self.objects[0].SetPaths(newpaths)
        else:
            obj = Sketch.PolyBezier(tuple(newpaths))
            self.set_objects([obj])
            
    def DragHandle(self, pa, pb, selection):
        try:
            inverse = self.trafo.inverse()
            pa, pb = map(inverse, (pa, pb))
            w, h = self.p + pb-pa
            if w<0: w = 0
            if h<0: h = 0
            self.p = Point(w,h)
            self.recompute()
        except SingularMatrix:
            pass

    def SetData(self, data):
        self.p = Point(data)

    def GetData(self):
        return list(self.p)

