###Sketch Config
#type = PluginCompound
#class_name = 'ConnectionHoriz'
#menu_text = 'Connection (horizontal)'
###End

import Sketch
from Sketch import Point, SingularMatrix
from Sketch.Graphics import handle
from lib.autoshapelib import AutoShape

# Unitvectors
ux = Point(1,0)
uy = Point(0,1)

class ConnectionHoriz(AutoShape):
    class_name = 'ConnectionHoriz'

    p1 = Point(0,0)
    p2 = Point(25,10)
    p3 = Point(50, 20)

    def GetSnapPoints(self):
        return self.p1, self.p3
    
    def GetHandles(self):
        handles = []
        for p in (self.p1, self.p2, self.p3):
            handles.append(handle.MakeNodeHandle(self.trafo(p)))
        return handles
                
    def recompute(self):
        new = Sketch.CreatePath()
        newpaths = [new]
        p1, p2, p3 = self.p1, self.p2, self.p3
        y1 = p1.y
        y2 = p2.y
        y3 = p3.y

        new.AppendLine(p1)
        new.AppendLine((p2[0], p1[1]))
        new.AppendLine((p2[0], p3[1]))
        new.AppendLine(p3)
        
        new.Transform(self.trafo)
        if self.objects:
            self.objects[0].SetPaths(newpaths)
        else:
            obj = Sketch.PolyBezier(tuple(newpaths))
            self.set_objects([obj])
            
    def DragHandle(self, pa, pb, selection):
        try:
            inverse = self.trafo.inverse()
            pa, pb = map(inverse, (pa, pb))
            delta = pb-pa

            p1, p2, p3 = self.p1, self.p2, self.p3

            x = p2.x
            if selection == 1:
                p1 = p1 + delta
            elif selection == 2:
                x = (p2 + delta).x
            elif selection == 3:
                p3 = p3 + delta

            y = (p3+p1).y /2.

            self.p1, self.p2, self.p3 = p1, Point(x,y), p3 
            self.recompute()
        except SingularMatrix:
            pass

    def SetData(self, data):
        undo = self.SetData, self.GetData()
        self.p1, self.p2, self.p3 = map(Point, data)
        return undo

    def GetData(self):
        return map(list, (self.p1, self.p2, self.p3))

