import sys
sys.path.append('/usr/local/lib/sketch/')
import Sketch
from Sketch import TrafoPlugin, _
from Sketch.Graphics.base import Editor
from Sketch.Graphics import handle

class GenericEditor(Editor):

    commands = []
    selection = 0

    def __init__(self, object):
        Editor.__init__(self, object)
        object.set_editor(self)
        self.transient_object = object.Duplicate()

    def Destroy(self):
        self.object.unset_editor(self)

    def ButtonDown(self, p, button, state):
        Editor.DragStart(self, p)
        self.transient_object = self.object.Duplicate()

    def ButtonUp(self, p, button, state):
        Editor.DragStop(self, p)
        return self.object.drag_handle(self.drag_start, p, self.selection)

    def MouseMove(self, p, state):
        Editor.MouseMove(self, p, state)
        self.transient_object = self.object.Duplicate()
        self.transient_object.drag_handle(self.drag_start, p, self.selection)

    def GetHandles(self):
        return self.object.GetHandles()

    def SelectPoint(self, p, rect, device, mode):
        return 0

    def SelectHandle(self, handle, mode):
        self.selection = handle.index+1

    def DrawDragged(self, device, partially):
        self.transient_object.DrawShape(device)

class AutoShape(TrafoPlugin):
    has_edit_mode = 1
    is_Group      = 1

    Ungroup = TrafoPlugin.GetObjects
    def __init__(self, data = None, trafo = None, duplicate = None, \
                 loading = 0):
        TrafoPlugin.__init__(self, trafo = trafo, duplicate = duplicate)

        if duplicate:
            data = duplicate.GetData()
            self.SetData(data)

        if loading:
            self.SetData(data)

        else:
            self.recompute()
                    
    editor = None
    def set_editor(self, editor):
        self.editor = editor
        
    def unset_editor(self, editor):
        if self.editor is editor:
            self.editor = None

    def Editor(self):
        return GenericEditor(self)

    def Info(self):
        return _("Autoshape `%s'") % self.class_name

    def SaveToFile(self, file):
        data = self.GetData()
        TrafoPlugin.SaveToFile(self, file, data,
                               self.trafo.coeff())

    def drag_handle(self, pa, pb, selection):
        data = self.GetData()
        self.DragHandle(pa, pb, selection)
        return self.set_data, data

    def set_data(self, data):
        undo = self.set_data, self.GetData()
        self.SetData(data)
        self.recompute()
        return undo

    # The following methods need to be implemented by Autoshapes
    def GetData(self):
        return None

    def SetData(self, data):
        pass

    def DragHandle(self, pa, pb, selection):
        pass
