from math import ceil
import sys, os, string, Tkinter
from Tkconstants import *
from Tkinter import StringVar, Entry, Label, Button, Frame, Text, Frame, \
     Label, StringVar, Radiobutton

import Sketch
from Sketch import Identity, TrafoPlugin, SimpleText, RegisterCommands, \
     PropertyStack, _, NullUndo, Scale, Translation, Group
from Sketch.Graphics.base  import Editor, RectangularPrimitive
from Sketch.Graphics.text import FactoryTextStyle
from Sketch.Graphics import handle
from Sketch.UI.command import AddCmd
from Sketch.Lib import encoding; iso_latin_1 = encoding.iso_latin_1

printable = ''
for n in range(len(iso_latin_1)):
    if iso_latin_1[n] != encoding.notdef:
        printable = printable + chr(n)


class Blocktext(TrafoPlugin, RectangularPrimitive):

    class_name = 'Blocktext'
    is_Group = 1
    is_Text  = 1
    has_font = 1
    has_fill = 1
    has_line = 0
    has_edit_mode = 1
    has_properties = 1

    has_edit_mode       = 1
    is_Text             = 1
    is_SimpleText       = 0
    is_TextBox          = 1
    is_curve            = 1
    is_clip             = 1
    has_font            = 1
    has_fill            = 1
    has_line            = 1

    Ungroup = TrafoPlugin.GetObjects
    commands = TrafoPlugin.commands

    def __init__(self, text='', trafo = None, properties = None,
                 duplicate = None, loading = 0):
        if properties is None:
            properties = PropertyStack(base=FactoryTextStyle())
            
        RectangularPrimitive.__init__(self, trafo,
                                      properties = properties,
                                      duplicate = duplicate)

        TrafoPlugin.__init__(self, trafo = trafo, duplicate = duplicate)        
               
        if loading:
            self.text = text
        elif duplicate:
            self.text = duplicate.text
        else:
            self.text = text
        self.rebuild()

    def open_shell(self):
         try:
             import pyshell
         except:
             return
         namespace = locals()
         namespace['self'] = self
         namespace['Sketch'] = Sketch
         namespace['doc'] = self.document
         import Tkinter
         tk = Tkinter.Tk()
         shell = pyshell.Shell(tk, namespace, width=40 )
         shell.pack(fill = 'both', expand = 1)

            
    def recompute(self):
        if self.needs_rebuild:
            self.rebuild()

    def rebuild(self):
        self.needs_rebuild = 0
        objects = []
        piecemap = self.piecemap = [] 
        height = 0
        y = 0

        props = self.properties
        x1, y1, x2, y2 = props.font.TextCoordBox('M', props.font_size)
        height = max(height, y2-y1)
        
        pieces = str(self.text).split('\n')
        i0 = 0
        for piece in pieces:
            y = y + height
            if len(piece)>0:
                line = Sketch.SimpleText(Identity, piece, properties=props)
                line.Transform(self.trafo(Translation(0, -y)))
                objects.append(line)
            else:
                line = None
            i1 = i0 + (len(piece))
            piecemap.append((i0, i1, y, line))
            i0 = i1+1

        if len(objects) == 0:
            line = Sketch.SimpleText(Identity, "<empty>", properties=props)
            line.Transform(self.trafo(Translation(0, -height)))
            objects.append(line)
        self.set_objects(objects)
        
    def _changed(self):
        TrafoPlugin._changed(self)
        self.rebuild()

    def SetParameters(self, params):
        return TrafoPlugin.SetParameters(self, params)

    def SetText(self, text, caret = None):
        if self.editor is not None:
            oldcaret = self.editor.Caret()
        else:
            oldcaret = 0
        undo = (self.SetText, self.text, oldcaret)
        self.text = text
        if caret is not None and self.editor is not None:
            self.editor.SetCaret(caret)
        self._changed()
        return undo

    def Properties(self):
        return self.properties
    
    def Font(self):
        return self.properties.font

    def FontSize(self):
        return self.properties.font_size

    def SaveToFile(self, file):
        TrafoPlugin.SaveToFile(self, file, self.text, self.trafo.coeff())
        
    editor = None
    def set_editor(self, editor):
        self.editor = editor

    def unset_editor(self, editor):
        if self.editor is editor:
            self.editor = None

    def Editor(self):
        return BlocktextEditor(self)
        
    def GetObjectHandle(self, *args):
        return []
    
    def AsBezier(self):
        bezier_objects = []
        for obj in self.objects:
            bezier_obj = obj.AsBezier()
            if bezier_obj is not None:
                bezier_objects.append(obj.AsBezier()) 
        if len(bezier_objects)>1:
            return Group(bezier_objects)
        else:
            return bezier_objects[0]

    def Paths(self):
        paths = []
        for obj in self.objects:
            _paths = obj.Paths()
            if _paths is not None:
                paths.extend(_paths)
        return tuple(paths)

RegisterCommands(Blocktext)


class BlocktextEditor(Editor):

    EditedClass = Blocktext
    commands = []

    def __init__(self, object):
        Editor.__init__(self, object)
        self.caret = 0
        object.set_editor(self)

    def Destroy(self):
        #self.document.AddAfterHandler(maybe_remove_text, (self.object,))
        self.object.unset_editor(self)

    def ButtonDown(self, p, button, state):
        Editor.DragStart(self, p)

    def ButtonUp(self, p, button, state):
        Editor.DragStop(self, p)

    def update_selection(self):
        # a bit ugly...
        if self.document is not None:
            self.document.queue_selection()

    def SetCaret(self, caret):
        if caret > len(self.text):
            caret = len(self.text)
        self.caret = caret

    def Caret(self):
        return self.caret

    def InsertCharacter(self, char):
        if len(char) == 1 and self.properties.font.IsPrintable(char):
            text = self.text;   caret = self.caret
            text = text[:caret] + char + text[caret:]
            return self.SetText(text, caret + 1)
        return NullUndo
    AddCmd(commands, InsertCharacter, '', key_stroke = tuple(printable), \
           invoke_with_keystroke = 1)

    def InsertNewline(self):
        text = self.text;   caret = self.caret
        text = text[:caret] + '\n' + text[caret:]    
        return self.SetText(text, caret + 1)
        
    AddCmd(commands, InsertNewline, '', key_stroke = 'Return')

    def DeleteCharBackward(self):
        if self.text and self.caret > 0:
            text = self.text; caret = self.caret
            text = text[:caret - 1] + text[caret:]
            return self.SetText(text, caret - 1)
        return NullUndo
    AddCmd(commands, DeleteCharBackward, '', key_stroke = 'BackSpace')

    def DeleteCharForward(self):
        if self.text and self.caret < len(self.text):
            text = self.text; caret = self.caret
            text = text[:caret] + text[caret + 1:]
            return self.SetText(text, caret)
        return NullUndo
    AddCmd(commands, DeleteCharForward, '', key_stroke = ('Delete', 'C-d'))

    def MoveForwardChar(self):
        if self.caret < len(self.text):
            self.SetCaret(self.caret + 1)
            self.update_selection()
        return NullUndo
    AddCmd(commands, MoveForwardChar, '', key_stroke = ('Right', 'C-f'))

    def MoveBackwardChar(self):
        if self.caret > 0:
            self.SetCaret(self.caret - 1)
            self.update_selection()
        return NullUndo
    AddCmd(commands, MoveBackwardChar, '', key_stroke = ('Left', 'C-b'))

    def MoveToBeginning(self):
        self.SetCaret(0)
        self.update_selection()
        return NullUndo
    AddCmd(commands, MoveToBeginning, '', key_stroke = ('Home', 'C-a'))
    
    def MoveToEnd(self):
        self.SetCaret(len(self.text))
        self.update_selection()
        return NullUndo
    AddCmd(commands, MoveToEnd, '', key_stroke = ('End', 'C-e'))

    def MoveToNextLine(self):
        caret = self.caret
        piecemap = self.piecemap
        i = 0
        for i in range(len(piecemap)):
            i0, i1, y, piece = piecemap[i]
            if i0 <= caret and caret <= i1: break
        if i < len(piecemap)-1:
            ni0, ni1 = piecemap[i+1][0:2]
            col = caret-i0
            caret = min(ni1, ni0+col )
            self.SetCaret(caret)
            return NullUndo
    AddCmd(commands, MoveToNextLine, '', key_stroke = ('Down', 'C-n'))

    def MoveToPrevLine(self):
        caret = self.caret
        piecemap = self.piecemap
        for i in range(len(piecemap)):
            i0, i1, y, piece = piecemap[i]
            if i0 <= caret and caret <= i1: break
            pi0 = i0
            pi1 = i1
        if i>0:
            col = caret-i0
            caret = min(pi1, pi0+col)
            self.SetCaret(caret)
        return NullUndo
    AddCmd(commands, MoveToPrevLine, '', key_stroke = ('Up', 'C-p'))
     
    def GetHandles(self):
        # returns the handle which is at position self.caret
        caret = self.caret
        for i0, i1, y, piece in self.piecemap:
            if i1 >= caret: break
            
        props = self.properties
        if piece is None:
            pos, up = props.font.TextCaretData(' ', 0, \
                                               props.font_size)
        else:
            pos, up = props.font.TextCaretData(piece.text, caret-i0, \
                                               props.font_size)
        trafo = self.trafo(Translation(0,-y))    
        pos = trafo(pos)
        up = trafo.DTransform(up)
        return [handle.MakeCaretHandle(pos, up)]

    def SelectPoint(self, p, rect, device, mode):
        dists = []
        i = 0
        for piece in self.objects:
            trafo = piece.trafo(Scale(piece.properties.font_size))
            trafo = trafo.inverse()
            p2 = trafo(p)
            pts = piece.properties.font.TypesetText(str(piece.text) + ' ')

            for j in range(len(pts)):
                dists.append((abs(pts[j]-p2), i))
                i = i+1

        caret = min(dists)[-1]
        self.SetCaret(caret)
        return 1
    
