###Sketch Config
#type = PluginCompound
#class_name = 'Smiley'
#menu_text = 'Smiley'
###End

import Sketch
from Sketch import Point, SingularMatrix, Scale, Translation
from Sketch.Graphics import handle
from lib.autoshapelib import AutoShape

class Smiley(AutoShape):
    class_name = 'Smiley'
    h = -10

    def GetObjectHandle(self, *args):
        return []

    def GetHandles(self):
        return [handle.MakeNodeHandle(self.trafo(Point(30, 10+self.h)))]
                
    def recompute(self):
        path = Sketch.CreatePath()
        newpaths = [path]
        h = self.h
        path.AppendLine((10,10))
        path.AppendBezier((15, 10+h/2.), (23, 10+h), (30,10+h))
        path.AppendBezier((37,10+h), (45, 10+h/2), (50, 10))
        path.Transform(self.trafo)
        if self.objects:
            self.objects[1].SetPaths(tuple(newpaths))
        else:
            skull = Sketch.Ellipse(Scale(45))
            skull.Transform(Translation(30, 33))
            mouth = Sketch.PolyBezier(tuple(newpaths))
            r_eye = Sketch.Ellipse(Scale(5))
            l_eye = Sketch.Ellipse(Scale(5))
            l_eye.Transform(Translation(15, 42))
            r_eye.Transform(Translation(45, 42))
            self.set_objects([skull, mouth, r_eye, l_eye])
            
    def DragHandle(self, pa, pb, selection):
        try:
            inverse = self.trafo.inverse()
            pa, pb = map(inverse, (pa, pb))
            h = self.h + (pb-pa)[1]
            h = min(15, h)
            h = max(-15, h)
            self.h = h
            self.recompute()
        except SingularMatrix:
            pass

    def SetData(self, data):
        self.h = data

    def GetData(self):
        return self.h

