###Sketch Config
#type = PluginCompound
#class_name = 'ConnectionVert'
#menu_text = 'Connection (vertical)'
###End

import Sketch
from Sketch import Point, SingularMatrix
from Sketch.Graphics import handle
from lib.autoshapelib import AutoShape

# Unitvectors
ux = Point(1,0)
uy = Point(0,1)

class ConnectionVert(AutoShape):
    class_name = 'ConnectionVert'
    p1 = Point(0,0)
    p2 = Point(25,10)
    p3 = Point(50, 20)

    def GetSnapPoints(self):
        return self.p1, self.p3
    
    def GetHandles(self):
        handles = []
        for p in (self.p1, self.p2, self.p3):
            handles.append(handle.MakeNodeHandle(self.trafo(p)))
        return handles
                
    def recompute(self):
        new = Sketch.CreatePath()
        newpaths = [new]
        p1, p2, p3 = self.p1, self.p2, self.p3

        new.AppendLine(p1)
        new.AppendLine((p1.x, p2.y))
        new.AppendLine((p3.x, p2.y))
        new.AppendLine(p3)
        
        new.Transform(self.trafo)
        if self.objects:
            self.objects[0].SetPaths(newpaths)
        else:
            obj = Sketch.PolyBezier()
            self.set_objects([obj])
            
    def DragHandle(self, pa, pb, selection):
        try:
            inverse = self.trafo.inverse()
            pa, pb = map(inverse, (pa, pb))
            delta = pb-pa
                         
            p1, p2, p3 = self.p1, self.p2, self.p3
            y = p2.y
            if selection == 1:
                p1 = p1 + delta
            elif selection == 2:
                y = (p2 + delta).y
            elif selection == 3:
                p3 = p3 + delta
            x = (p3+p1).x /2.

            self.p1, self.p2, self.p3 = p1, Point(x,y), p3 
            self.recompute()
        except SingularMatrix:
            pass

    def SetData(self, data):
        self.p1, self.p2, self.p3 = map(Point, data)

    def GetData(self):
        return map(list, (self.p1, self.p2, self.p3))

